
; Patch tcpip32.dll to make malloc/free calls MT-safe and SMP-safe.

; Tue Aug 16 06:01:00 2011 Steven H. Levine <steve53 at eartlink dot net>

; Some of the malloc and free calls in tcpip32.dll are not MT-safe.
; This was discovered in the course of analyzing Seamonkey traps on SMP systems.
; For unknown reasons, IBM did not link this DLL with an MT-safe runtime.
; IBM was aware of the issue and implemented some naive fixes, but the
; fixes did not handle all the malloc/free calls.  The fixes handled only
; calls made directly by the tcpip32 code.

; This patch makes all the malloc/free calls MT-safe and SMP-safe.
; It also updates the bldlevel string to MPTS 6.01a and corrects it to
; display with bldlevel.exe

;==============================
; How to apply this patch file.
;==============================

; Obtain a copy of tcpip32.dll from an eCS 2.x distribution or elsewhere.
; You need the tcpip32.dll that reports itself as

; 5-21-04  11:15          98,611      0  tcpip32.dll

; with the MD5 hash

; 240b8724966953475b7bad7f122e4059 *tcpip32.dll

; The level information will not display with bldlevel.exe, so you need
; to use the MD5 hash or

;  lxlite -c:exehdr tcpip32.dll
  
; to be sure you have the right file.

; Unpack the file with the lxlite command

;  lxlite -X- -B- tcpip32.dll

; Apply the patch with the patch command

;  patch /a tcpip32_mtsafe-malloc-patch-shl.pat

; Be sure to use the IBM patch.exe.  The GNU patch.exe will not work.

; Pack the resulting file with the lxlite command.

;  lxlite -X+ -B- tcpip32.dll

; The repacked DLL should report itself as

; 8-16-11   6:01          87,504      0  tcpip32.dll

; with bldlevel

;  Signature:       @#IBM:6.01a#@ patched 2011/08/16 06:01:01 on TCPBLDSRVR - MPTS 6.01-TCP/IP 4.3 for OS/2 - 32 BIT SOCKETS
;  Vendor:          IBM
;  Revision:        6.01
;  File Version:    6.1
;  Description:     patched 2011/08/16 06:01:01 on TCPBLDSRVR - MPTS 6.01-TCP/IP 4.3 for OS/2 - 32

; with MD5 hash

;  ba84ca10398803096af52facd7651b75 *tcpip32.dll

;==========================================
; Patch instructions and development notes.
;==========================================

file tcpip32.dll

; base offset for disassembly = 10000

;==================
;== malloc calls ==
;==================

; All address values are 32-bit hex
; Object numbers and indexes are decimal

; from
; 102C0 malloc          proc near
; to
; 2AEF0 safeMalloc      proc near

;============
; lkupHandle:
; 142D8 E8 E3 BF FF FF   call    malloc
;   ffffbfe3

;   142d8 + 5 = 142DD
;   142DD  + ffffbfe3 = 102C0 (malloc)
;   2AEF0 - 142DD = 16C13
;   SafeMalloc
;   142DD + 16C13 = 2AEF0 (safeMalloc)
;     13 6c 01 00

;   file offset
;   1428d - 10000 = 42d8 = object 1 index 5 offset 2d8
;      Index  FileOffs  Size  Attribute 
;       00000005  00008400  1000    Valid   

;   8400 + 2d8 = 86d8h

; from                            vv
; 86D0  F9 8B D8 B8  08 00 00 00  E8 E3 BF FF  FF 8B F8 85
; to
; 86D0  F9 8B D8 B8  08 00 00 00  E8 13 6C 01  00 8B F8 85

VER 86D0 F98BD8B808000000E8E3BFFFFF8BF885
CHA 86D0 F98BD8B808000000E8136C01008BF885

;==========
; __dofmto:
; 11867 E8 54 EA FF FF   call    malloc          ; work buffer
;   ffffea54

;   11867 + 5 = 1186C
;   1186C  + ffffea54 = 102C0 (malloc)
;   0002AEF0 - 1186C = 19684
;   SafeMalloc
;   1186C + 19684 = 2AEF0 (safeMalloc)
;     84 96 01 00

;   file offset
;   11867 - 10000 = 1867 = object 1 index 2 offset 867
;      Index  FileOffs  Size  Attribute 
;       00000002  00005400  1000    Valid   

;   5400 + 867 = 5c67

; from                       vv
; 5C60 8B DA B8 F0  01 00 00 E8  54 EA FF FF  8B D3 89 84
; to
; 5C60 8B DA B8 F0  01 00 00 E8  84 96 01 00  8B D3 89 84

VER 5C60 8BDAB8F0010000E854EAFFFF8BD38984
CHA 5C60 8BDAB8F0010000E8849601008BD38984

;===========
; _bufprint:
; 14345 E8 76 BF FF FF   call    malloc   ; work buffer
;   FFFFBF76

;   14345 + 5 = 1434A
;   1434A  + FFFFBF76 = 102C0 (malloc)
;   2AEF0 - 1434A = 16BA6
;   SafeMalloc
;   1434A + 16BA6 = 2AEF0 (safeMalloc)
;     A6 6B 01 00

;   file offset
;   14345 - 10000 = 4345 = object 1 index 5 offset 345
;      Index  FileOffs  Size  Attribute 
;       00000005  00008400  1000    Valid   

;   8400 + 345 = 8745h

; from                  vv
; 8740  B8 E0 03 00  00 E8 76 BF  FF FF A3 60  11 04 00 85
; to
; 8740  B8 E0 03 00  00 E8 A6 6B 01 00  A3 60  11 04 00 85

VER 8740 B8E0030000E876BFFFFFA36011040085
CHA 8740 B8E0030000E8A66B0100A36011040085

;================
;== free calls ==
;================

; from
; 1028C free            proc near
; to
; 2AF40 safeFree        proc near

;================
; _delete_handle:
; 14279 E9 0E C0 FF FF   jmp     free   ; eax
;   ffffc00e

;   14279 + 5 = 1427E
;   1427E + ffffc00e = 1028C (free)
;   SafeFree
;   0002AF40 - 1427E = 16CC2
;   1427E + 16CC2 = 2AF40 (safeFree)
;     C2 6C 01 00

;   file offset
;   14279 - 10000 = 4279 = object 1 index 5 offset 279
;      Index  FileOffs  Size  Attribute 
;       00000005  00008400  1000    Valid   

;   8400 + 279 = 8679

; from                              vv
; 8670 75 0C 5E 8B  48 04 89 0F  5F E9 0E C0  FF FF 8B 3F
; to
; 8670 75 0C 5E 8B  48 04 89 0F  5F E9 C2 6C  01 00 8B 3F

VER 8670 750C5E8B4804890F5FE90EC0FFFF8B3F
CHA 8670 750C5E8B4804890F5FE9C26C01008B3F

;==========
; __dofmto:
; 11D1B E8 6C E5 FF FF   call    free
;    FFFFE56C

;   11D1B + 5 = 11D20
;   11D20 + FFFFE56C = 1028C (free)
;   SafeFree
;   2AF40 - 11D20 = 19220
;   11D20 + 19220 = 2AF40 (safeFree)
;     20 92 01 00

;   file offset
;   11D1B - 10000 = 1D1B = object 1 index 2 offset D1B
;      Index  FileOffs  Size  Attribute 
;       00000002  00005400  1000    Valid   

;   5400 + D1B = 611B

; from                                         vv
; 6110  00 00 75 0C  8B 84 24 94  03 00 00 E8  6C E5 FF FF
; to
; 6110  00 00 75 0C  8B 84 24 94  03 00 00 E8  20 92 01 00

VER 6110 0000750C8B842494030000E86CE5FFFF
CHA 6110 0000750C8B842494030000E820920100

;========
; strdup:
; 106C5  call    malloc

; No patch needed - all strdup calls already wrapped by safeStrdup

;==========
; bldlevel:
;==========

; IBM used a malformed bldlevel that bldlevel.exe will not display
; @#IBM:MPTS 6.01#@##built 14:11:09 Sun May 18 2003 -- On TCPBLDSRVR;0.1@@ TCP/IP for OS/2: 32 BIT SOCKETS

; 20440  40 23 49 42  4D 3A 4D 50  54 53 20 36  2E 30 31 23  @#IBM:MPTS 6.01#
; 20450  40 23 23 62  75 69 6C 74  20 31 34 3A  31 31 3A 30  @##built 14:11:0
; 20460  39 20 53 75  6E 20 4D 61  79 20 31 38  20 32 30 30  9 Sun May 18 200
; 20470  33 20 2D 2D  20 4F 6E 20  54 43 50 42  4C 44 53 52  3 -- On TCPBLDSR
; 20480  56 52 3B 30  2E 31 40 40  20 54 43 50  2F 49 50 20  VR;0.1@@ TCP/IP
; 20490  66 6F 72 20  4F 53 2F 32  3A 20 33 32  20 42 49 54  for OS/2: 32 BIT
; 204A0  20 53 4F 43  4B 45 54 53                             SOCKETS

VER 20440 402349424D3A4D50545320362E303123
VER 20450 4023236275696C742031343A31313A30
VER 20460 392053756E204D617920313820323030
VER 20470 33202D2D204F6E20544350424C445352
VER 20480 56523B302E314040205443502F495020
VER 20490 666F72204F532F323A20333220424954
VER 204A0 20534F434B455453

; replaced with
; @#IBM:6.01a#@ patched 2011/08/16 06:01:01 on TCPBLDSRVR - MPTS 6.01-TCP/IP 4.3 for OS/2 - 32 BIT SOCKETS


; 20440  40 23 49 42  4D 3A 36 2E  30 31 61 23  40 20 70 61  @#IBM:6.01a#@ pa
; 20450  74 63 68 65  64 20 32 30  31 31 2F 30  38 2F 31 36  tched 2011/08/16
; 20460  20 30 36 3A  30 31 3A 30  31 20 6F 6E  20 54 43 50   06:01:01 on TCP
; 20470  42 4C 44 53  52 56 52 20  2D 20 4D 50  54 53 20 36  BLDSRVR - MPTS 6
; 20480  2E 30 31 2D  54 43 50 2F  49 50 20 34  2E 33 20 66  .01-TCP/IP 4.3 f
; 20490  6F 72 20 4F  53 2F 32 20  2D 20 33 32  20 42 49 54  or OS/2 - 32 BIT
; 204A0  20 53 4F 43  4B 45 54 53                             SOCKETS

CHA 20440 402349424D3A362E3031612340207061
CHA 20450 746368656420323031312F30382F3136
CHA 20460 2030363A30313A3031206F6E20544350
CHA 20470 424C4453525652202D204D5054532036
CHA 20480 2E30312D5443502F495020342E332066
CHA 20490 6F72204F532F32202D20333220424954
CHA 204A0 20534F434B455453

;======
; Notes
;======

; Remaining realloc _umalloc _ucalloc _ufree calls are OK because all
; are implicitly wrapped by safe... functions.

; The end
